@@ -514,8 +514,9 @@ JVM_handle_linux_signal(int sig,
 
 #ifdef CAN_SHOW_REGISTERS_ON_ASSERT
   if ((sig == SIGSEGV || sig == SIGBUS) && info != NULL && info->si_addr == g_assert_poison) {
-    handle_assert_poison_fault(ucVoid, info->si_addr);
-    return 1;
+    if (handle_assert_poison_fault(ucVoid, info->si_addr)) {
+      return 1;
+    }
   }
 #endif
 
@@ -552,7 +553,7 @@ JVM_handle_linux_signal(int sig,
       return 1;
     }
 
-    if (checkPrefetch(uc, pc)) {
+    if ((sig == SIGSEGV || sig == SIGBUS) && checkPrefetch(uc, pc)) {
       return 1;
     }
 
