@@ -293,7 +293,7 @@ class ClassLoaderData : public CHeapObj<mtClass> {
   bool keep_alive() const       { return _keep_alive > 0; }
 
   oop holder_phantom() const;
-  void classes_do(void f(Klass*));
+  void classes_do(void f(Klass* const));
   void loaded_classes_do(KlassClosure* klass_closure);
   void classes_do(void f(InstanceKlass*));
   void methods_do(void f(Method*));
@@ -419,6 +419,10 @@ class ClassLoaderData : public CHeapObj<mtClass> {
   const char* loader_name_and_id() const;
   Symbol* name_and_id() const { return _name_and_id; }
 
+  unsigned identity_hash() const {
+    return (unsigned)((uintptr_t)this >> 3);
+  }
+
   JFR_ONLY(DEFINE_TRACE_ID_METHODS;)
 };
 
