@@ -564,8 +564,6 @@ void GenCollectedHeap::do_collection(bool           full,
 
   ClearedAllSoftRefs casr(do_clear_all_soft_refs, soft_ref_policy());
 
-  const size_t metadata_prev_used = MetaspaceUtils::used_bytes();
-
   print_heap_before_gc();
 
   {
@@ -590,6 +588,7 @@ void GenCollectedHeap::do_collection(bool           full,
 
     size_t young_prev_used = _young_gen->used();
     size_t old_prev_used = _old_gen->used();
+    const metaspace::MetaspaceSizesSnapshot prev_meta_sizes;
 
     bool run_verification = total_collections() >= VerifyGCStartAt;
 
@@ -650,7 +649,7 @@ void GenCollectedHeap::do_collection(bool           full,
     complete = complete || collected_old;
 
     print_heap_change(young_prev_used, old_prev_used);
-    MetaspaceUtils::print_metaspace_change(metadata_prev_used);
+    MetaspaceUtils::print_metaspace_change(prev_meta_sizes);
 
     // Adjust generation sizes.
     if (collected_old) {
