@@ -31,6 +31,7 @@
 #include "oops/method.hpp"
 #include "oops/oop.hpp"
 #include "runtime/atomic.hpp"
+#include "runtime/deoptimization.hpp"
 #include "utilities/align.hpp"
 #include "utilities/copy.hpp"
 #if INCLUDE_JVMCI
@@ -1983,7 +1984,7 @@ public:
 
   // Whole-method sticky bits and flags
   enum {
-    _trap_hist_limit    = 24 JVMCI_ONLY(+5),   // decoupled from Deoptimization::Reason_LIMIT
+    _trap_hist_limit    = Deoptimization::Reason_TRAP_HISTORY_LENGTH,
     _trap_hist_mask     = max_jubyte,
     _extra_data_count   = 4     // extra DataLayout headers, for trap history
   }; // Public flag values
@@ -1999,6 +2000,7 @@ public:
     uint _nof_overflow_traps;         // trap count, excluding _trap_hist
     union {
       intptr_t _align;
+      // JVMCI separates trap history for OSR compilations from normal compilations
       u1 _array[JVMCI_ONLY(2 *) MethodData::_trap_hist_limit];
     } _trap_hist;
 
@@ -2021,14 +2023,14 @@ public:
 
     // Return (uint)-1 for overflow.
     uint trap_count(int reason) const {
-      assert((uint)reason < JVMCI_ONLY(2*) _trap_hist_limit, "oob");
+      assert((uint)reason < ARRAY_SIZE(_trap_hist._array), "oob");
       return (int)((_trap_hist._array[reason]+1) & _trap_hist_mask) - 1;
     }
 
     uint inc_trap_count(int reason) {
       // Count another trap, anywhere in this method.
       assert(reason >= 0, "must be single trap");
-      assert((uint)reason < JVMCI_ONLY(2*) _trap_hist_limit, "oob");
+      assert((uint)reason < ARRAY_SIZE(_trap_hist._array), "oob");
       uint cnt1 = 1 + _trap_hist._array[reason];
       if ((cnt1 & _trap_hist_mask) != 0) {  // if no counter overflow...
         _trap_hist._array[reason] = cnt1;
