@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2016, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2018, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -23,12 +23,13 @@
  * questions.
  */
 
-#import <JavaNativeFoundation/JavaNativeFoundation.h>
 #include <Carbon/Carbon.h>
 #import "CMenuItem.h"
 #import "CMenu.h"
 #import "AWTEvent.h"
+#import "AWTWindow.h"
 #import "ThreadUtilities.h"
+#import "JNIUtilities.h"
 
 #import "java_awt_Event.h"
 #import "java_awt_event_KeyEvent.h"
@@ -68,7 +69,7 @@
 - (void)handleAction:(NSMenuItem *)sender {
     AWT_ASSERT_APPKIT_THREAD;
     JNIEnv *env = [ThreadUtilities getJNIEnv];
-    JNF_COCOA_ENTER(env);
+    JNI_COCOA_ENTER(env);
     
     // If we are called as a result of user pressing a shortcut, do nothing,
     // because AVTView has already sent corresponding key event to the Java
@@ -81,14 +82,14 @@
     // means we have to handle it here.
     NSEvent *currEvent = [[NSApplication sharedApplication] currentEvent];
     if (fIsCheckbox) {
-        static JNF_CLASS_CACHE(jc_CCheckboxMenuItem, "sun/lwawt/macosx/CCheckboxMenuItem");
-        static JNF_MEMBER_CACHE(jm_ckHandleAction, jc_CCheckboxMenuItem, "handleAction", "(Z)V");
+        DECLARE_CLASS(jc_CCheckboxMenuItem, "sun/lwawt/macosx/CCheckboxMenuItem");
+        DECLARE_METHOD(jm_ckHandleAction, jc_CCheckboxMenuItem, "handleAction", "(Z)V");
         
         // Send the opposite of what's currently checked -- the action
         // indicates what state we're going to.
         NSInteger state = [sender state];
         jboolean newState = (state == NSOnState ? JNI_FALSE : JNI_TRUE);
-        JNFCallVoidMethod(env, fPeer, jm_ckHandleAction, newState);
+        (*env)->CallVoidMethod(env, fPeer, jm_ckHandleAction, newState);
     }
     else {
         if ([currEvent type] == NSKeyDown) {
@@ -111,35 +112,38 @@
                 }
                 eventKey = [NSString stringWithCharacters: &newChar length: 1];
             }
+            // The action event can be ignored only if the key window is an AWT window.
+            // Otherwise, the action event is the only notification and must be processed.
             NSWindow *keyWindow = [NSApp keyWindow];
-            if (keyWindow != nil) {
+            if (keyWindow != nil && [AWTWindow isAWTWindow: keyWindow]) {
                 return;
             }
-		}
-		
-        static JNF_CLASS_CACHE(jc_CMenuItem, "sun/lwawt/macosx/CMenuItem");
-        static JNF_MEMBER_CACHE(jm_handleAction, jc_CMenuItem, "handleAction", "(JI)V"); // AWT_THREADING Safe (event)
+        }
+        
+        DECLARE_CLASS(jc_CMenuItem, "sun/lwawt/macosx/CMenuItem");
+        DECLARE_METHOD(jm_handleAction, jc_CMenuItem, "handleAction", "(JI)V"); // AWT_THREADING Safe (event)
 
         NSUInteger modifiers = [currEvent modifierFlags];
         jint javaModifiers = NsKeyModifiersToJavaModifiers(modifiers, NO);
 
-        JNFCallVoidMethod(env, fPeer, jm_handleAction, UTC(currEvent), javaModifiers); // AWT_THREADING Safe (event)
+        (*env)->CallVoidMethod(env, fPeer, jm_handleAction, UTC(currEvent), javaModifiers); // AWT_THREADING Safe (event)
     }
-    JNF_COCOA_EXIT(env);
-	
+    CHECK_EXCEPTION();
+    JNI_COCOA_EXIT(env);
+    
 }
 
 - (void) setJavaLabel:(NSString *)theLabel shortcut:(NSString *)theKeyEquivalent modifierMask:(jint)modifiers {
-    
+
     NSUInteger modifierMask = 0;
-    
+
     if (![theKeyEquivalent isEqualToString:@""]) {
         // Force the key equivalent to lower case if not using the shift key.
         // Otherwise AppKit will draw a Shift glyph in the menu.
         if ((modifiers & java_awt_event_KeyEvent_SHIFT_MASK) == 0) {
             theKeyEquivalent = [theKeyEquivalent lowercaseString];
         }
-        
+
         // Hack for the question mark -- SHIFT and / means use the question mark.
         if ((modifiers & java_awt_event_KeyEvent_SHIFT_MASK) != 0 &&
             [theKeyEquivalent isEqualToString:@"/"])
@@ -147,10 +151,10 @@
             theKeyEquivalent = @"?";
             modifiers &= ~java_awt_event_KeyEvent_SHIFT_MASK;
         }
-        
+
         modifierMask = JavaModifiersToNsKeyModifiers(modifiers, NO);
     }
-    
+
     [ThreadUtilities performOnMainThreadWaiting:YES block:^(){
         [fMenuItem setKeyEquivalent:theKeyEquivalent];
         [fMenuItem setKeyEquivalentModifierMask:modifierMask];
@@ -159,14 +163,14 @@
 }
 
 - (void) setJavaImage:(NSImage *)theImage {
-    
+
     [ThreadUtilities performOnMainThreadWaiting:NO block:^(){
         [fMenuItem setImage:theImage];
     }];
 }
 
 - (void) setJavaToolTipText:(NSString *)theText {
-    
+
     [ThreadUtilities performOnMainThreadWaiting:NO block:^(){
         [fMenuItem setToolTip:theText];
     }];
@@ -174,11 +178,11 @@
 
 
 - (void)setJavaEnabled:(BOOL) enabled {
-    
+
     [ThreadUtilities performOnMainThreadWaiting:NO block:^(){
         @synchronized(self) {
             fIsEnabled = enabled;
-            
+
             // Warning:  This won't work if the parent menu is disabled.
             // See [CMenu syncFromJava]. We still need to call it here so
             // the NSMenuItem itself gets properly updated.
@@ -188,7 +192,7 @@
 }
 
 - (BOOL)isEnabled {
-    
+
     BOOL enabled = NO;
     @synchronized(self) {
         enabled = fIsEnabled;
@@ -198,7 +202,7 @@
 
 
 - (void)setJavaState:(BOOL)newState {
-    
+
     [ThreadUtilities performOnMainThreadWaiting:NO block:^(){
         [fMenuItem setState:(newState ? NSOnState : NSOffState)];
     }];
@@ -209,7 +213,7 @@
     [fMenuItem setTarget:nil];
     [fMenuItem release];
     fMenuItem = nil;
-    
+
     [super dealloc];
 }
 
@@ -234,7 +238,7 @@
 /** Convert a Java keycode for SetMenuItemCmd */
 static unichar AWTKeyToMacShortcut(jint awtKey, BOOL doShift) {
     unichar macKey = 0;
-    
+
     if ((awtKey >= java_awt_event_KeyEvent_VK_0 && awtKey <= java_awt_event_KeyEvent_VK_9) ||
         (awtKey >= java_awt_event_KeyEvent_VK_A && awtKey <= java_awt_event_KeyEvent_VK_Z))
     {
@@ -251,43 +255,43 @@ static unichar AWTKeyToMacShortcut(jint awtKey, BOOL doShift) {
         switch (awtKey) {
             case java_awt_event_KeyEvent_VK_BACK_QUOTE      : macKey = '`'; break;
             case java_awt_event_KeyEvent_VK_QUOTE           : macKey = '\''; break;
-                
+
             case java_awt_event_KeyEvent_VK_ESCAPE          : macKey = 0x1B; break;
             case java_awt_event_KeyEvent_VK_SPACE           : macKey = ' '; break;
             case java_awt_event_KeyEvent_VK_PAGE_UP         : macKey = NSPageUpFunctionKey; break;
             case java_awt_event_KeyEvent_VK_PAGE_DOWN       : macKey = NSPageDownFunctionKey; break;
             case java_awt_event_KeyEvent_VK_END             : macKey = NSEndFunctionKey; break;
             case java_awt_event_KeyEvent_VK_HOME            : macKey = NSHomeFunctionKey; break;
-                
+
             case java_awt_event_KeyEvent_VK_LEFT            : macKey = NSLeftArrowFunctionKey; break;
             case java_awt_event_KeyEvent_VK_UP              : macKey = NSUpArrowFunctionKey; break;
             case java_awt_event_KeyEvent_VK_RIGHT           : macKey = NSRightArrowFunctionKey; break;
             case java_awt_event_KeyEvent_VK_DOWN            : macKey = NSDownArrowFunctionKey; break;
-                
+
             case java_awt_event_KeyEvent_VK_COMMA           : macKey = ','; break;
-                
+
                 // Mac OS doesn't distinguish between the two '-' keys...
             case java_awt_event_KeyEvent_VK_MINUS           :
             case java_awt_event_KeyEvent_VK_SUBTRACT        : macKey = '-'; break;
-                
+
                 // or the two '.' keys...
             case java_awt_event_KeyEvent_VK_DECIMAL         :
             case java_awt_event_KeyEvent_VK_PERIOD          : macKey = '.'; break;
-                
+
                 // or the two '/' keys.
             case java_awt_event_KeyEvent_VK_DIVIDE          :
             case java_awt_event_KeyEvent_VK_SLASH           : macKey = '/'; break;
-                
+
             case java_awt_event_KeyEvent_VK_SEMICOLON       : macKey = ';'; break;
             case java_awt_event_KeyEvent_VK_EQUALS          : macKey = '='; break;
-                
+
             case java_awt_event_KeyEvent_VK_OPEN_BRACKET    : macKey = '['; break;
             case java_awt_event_KeyEvent_VK_BACK_SLASH      : macKey = '\\'; break;
             case java_awt_event_KeyEvent_VK_CLOSE_BRACKET   : macKey = ']'; break;
-                
+
             case java_awt_event_KeyEvent_VK_MULTIPLY        : macKey = '*'; break;
             case java_awt_event_KeyEvent_VK_ADD             : macKey = '+'; break;
-                
+
             case java_awt_event_KeyEvent_VK_HELP            : macKey = NSHelpFunctionKey; break;
             case java_awt_event_KeyEvent_VK_TAB             : macKey = NSTabCharacter; break;
             case java_awt_event_KeyEvent_VK_ENTER           : macKey = NSNewlineCharacter; break;
@@ -327,24 +331,24 @@ Java_sun_lwawt_macosx_CMenuItem_nativeSetLabel
  jlong menuItemObj, jstring label,
  jchar shortcutKey, jint shortcutKeyCode, jint mods)
 {
-    JNF_COCOA_ENTER(env);
-    NSString *theLabel = JNFJavaToNSString(env, label);
+    JNI_COCOA_ENTER(env);
+    NSString *theLabel = JavaStringToNSString(env, label);
     NSString *theKeyEquivalent = nil;
     unichar macKey = shortcutKey;
-    
+
     if (macKey == 0) {
         macKey = AWTKeyToMacShortcut(shortcutKeyCode, (mods & java_awt_event_KeyEvent_SHIFT_MASK) != 0);
     }
-    
+
     if (macKey != 0) {
         unichar equivalent[1] = {macKey};
         theKeyEquivalent = [NSString stringWithCharacters:equivalent length:1];
     } else {
         theKeyEquivalent = @"";
     }
-    
+
     [((CMenuItem *)jlong_to_ptr(menuItemObj)) setJavaLabel:theLabel shortcut:theKeyEquivalent modifierMask:mods];
-    JNF_COCOA_EXIT(env);
+    JNI_COCOA_EXIT(env);
 }
 
 /*
@@ -356,10 +360,10 @@ JNIEXPORT void JNICALL
 Java_sun_lwawt_macosx_CMenuItem_nativeSetTooltip
 (JNIEnv *env, jobject peer, jlong menuItemObj, jstring tooltip)
 {
-    JNF_COCOA_ENTER(env);
-    NSString *theTooltip = JNFJavaToNSString(env, tooltip);
+    JNI_COCOA_ENTER(env);
+    NSString *theTooltip = JavaStringToNSString(env, tooltip);
     [((CMenuItem *)jlong_to_ptr(menuItemObj)) setJavaToolTipText:theTooltip];
-    JNF_COCOA_EXIT(env);
+    JNI_COCOA_EXIT(env);
 }
 
 /*
@@ -371,9 +375,9 @@ JNIEXPORT void JNICALL
 Java_sun_lwawt_macosx_CMenuItem_nativeSetImage
 (JNIEnv *env, jobject peer, jlong menuItemObj, jlong image)
 {
-    JNF_COCOA_ENTER(env);
+    JNI_COCOA_ENTER(env);
     [((CMenuItem *)jlong_to_ptr(menuItemObj)) setJavaImage:(NSImage*)jlong_to_ptr(image)];
-    JNF_COCOA_EXIT(env);
+    JNI_COCOA_EXIT(env);
 }
 
 /*
@@ -385,12 +389,12 @@ JNIEXPORT jlong JNICALL
 Java_sun_lwawt_macosx_CMenuItem_nativeCreate
 (JNIEnv *env, jobject peer, jlong parentCMenuObj, jboolean isSeparator)
 {
-    
+
     __block CMenuItem *aCMenuItem = nil;
     BOOL asSeparator = (isSeparator == JNI_TRUE) ? YES: NO;
     CMenu *parentCMenu = (CMenu *)jlong_to_ptr(parentCMenuObj);
-    JNF_COCOA_ENTER(env);
-    
+    JNI_COCOA_ENTER(env);
+
     jobject cPeerObjGlobal = (*env)->NewGlobalRef(env, peer);
 
     [ThreadUtilities performOnMainThreadWaiting:YES block:^(){
@@ -398,17 +402,17 @@ Java_sun_lwawt_macosx_CMenuItem_nativeCreate
                                          asSeparator: asSeparator];
         // the CMenuItem is released in CMenuComponent.dispose()
     }];
-    
+
     if (aCMenuItem == nil) {
         return 0L;
     }
-    
+
     // and add it to the parent item.
     [parentCMenu addJavaMenuItem: aCMenuItem];
-    
+
     // setLabel will be called after creation completes.
-    
-    JNF_COCOA_EXIT(env);
+
+    JNI_COCOA_EXIT(env);
     return ptr_to_jlong(aCMenuItem);
 }
 
@@ -421,10 +425,10 @@ JNIEXPORT void JNICALL
 Java_sun_lwawt_macosx_CMenuItem_nativeSetEnabled
 (JNIEnv *env, jobject peer, jlong menuItemObj, jboolean enable)
 {
-    JNF_COCOA_ENTER(env);
+    JNI_COCOA_ENTER(env);
     CMenuItem *item = (CMenuItem *)jlong_to_ptr(menuItemObj);
     [item setJavaEnabled: (enable == JNI_TRUE)];
-    JNF_COCOA_EXIT(env);
+    JNI_COCOA_EXIT(env);
 }
 
 /*
@@ -436,10 +440,10 @@ JNIEXPORT void JNICALL
 Java_sun_lwawt_macosx_CCheckboxMenuItem_nativeSetState
 (JNIEnv *env, jobject peer, jlong menuItemObj, jboolean state)
 {
-    JNF_COCOA_ENTER(env);
+    JNI_COCOA_ENTER(env);
     CMenuItem *item = (CMenuItem *)jlong_to_ptr(menuItemObj);
     [item setJavaState: (state == JNI_TRUE)];
-    JNF_COCOA_EXIT(env);
+    JNI_COCOA_EXIT(env);
 }
 
 /*
@@ -451,8 +455,8 @@ JNIEXPORT void JNICALL
 Java_sun_lwawt_macosx_CCheckboxMenuItem_nativeSetIsCheckbox
 (JNIEnv *env, jobject peer, jlong menuItemObj)
 {
-    JNF_COCOA_ENTER(env);
+    JNI_COCOA_ENTER(env);
     CMenuItem *item = (CMenuItem *)jlong_to_ptr(menuItemObj);
     [item setIsCheckbox];
-    JNF_COCOA_EXIT(env);
+    JNI_COCOA_EXIT(env);
 }
