@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2007, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1999, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -32,6 +32,7 @@
 #define _Included_SurfaceData
 
 #include <jni.h>
+#include <limits.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -53,6 +54,14 @@ typedef struct {
 
 #define SD_RASINFO_PRIVATE_SIZE         64
 
+#define UNSAFE_TO_ADD(a, b) \
+    (((a >= 0) && (b >= 0) && (a > (INT_MAX - b))) || \
+     ((a < 0) && (b < 0) && (a < (INT_MIN - b)))) \
+
+#define UNSAFE_TO_SUB(a, b) \
+    (((b >= 0) && (a < 0) && (a < (INT_MIN + b))) || \
+     ((b < 0) && (a >= 0) && (a > (INT_MAX + b)))) \
+
 /*
  * The SurfaceDataRasInfo structure is used to pass in and return various
  * pieces of information about the destination drawable.  In particular:
@@ -129,9 +138,9 @@ typedef struct {
  * array of bytes indexed by RxGxB where each component is reduced to 5
  * bits of precision before indexing.
  *
- *      char *redErrTable;
- *      char *grnErrTable;
- *      char *bluErrTable;
+ *      signed char *redErrTable;
+ *      signed char *grnErrTable;
+ *      signed char *bluErrTable;
  * [Requires SD_LOCK_INVCOLOR]
  * Pointers to the beginning of the ordered dither color error tables
  * for the colormap.  The error tables are formatted as an 8x8 array
@@ -159,9 +168,9 @@ typedef struct {
     unsigned int        lutSize;                /* # colors in colormap */
     jint                *lutBase;               /* Pointer to colormap[0] */
     unsigned char       *invColorTable;         /* Inverse color table */
-    char                *redErrTable;           /* Red ordered dither table */
-    char                *grnErrTable;           /* Green ordered dither table */
-    char                *bluErrTable;           /* Blue ordered dither table */
+    signed char         *redErrTable;           /* Red ordered dither table */
+    signed char         *grnErrTable;           /* Green ordered dither table */
+    signed char         *bluErrTable;           /* Blue ordered dither table */
     int                 *invGrayTable;          /* Inverse gray table */
     int                 representsPrimaries;    /* whether cmap represents primary colors */
     union {
@@ -649,7 +658,8 @@ SurfaceData_IntersectBlitBounds(SurfaceDataBounds *Abounds,
  * Subclasses of SurfaceData should call this function instead of allocating
  * the memory directly.
  */
-SurfaceDataOps *SurfaceData_InitOps(JNIEnv *env, jobject sData, int opsSize);
+JNIEXPORT SurfaceDataOps * JNICALL
+SurfaceData_InitOps(JNIEnv *env, jobject sData, int opsSize);
 
 /*
  * This function invokes the ops-specific disposal function.
