@@ -723,9 +723,6 @@ public class LogParser extends DefaultHandler implements ErrorHandler {
      */
     String type(String id) {
         String result = types.get(id);
-        if (result == null) {
-            reportInternalError(id);
-        }
         String remapped = type2printableMap.get(result);
         if (remapped != null) {
             return remapped;
@@ -975,6 +972,12 @@ public class LogParser extends DefaultHandler implements ErrorHandler {
                 // track of it but don't add it to the list yet.
                 methodHandleSite = site;
             }
+        } else if (qname.equals("intrinsic")) {
+            String id = atts.getValue("id");
+            assert id != null : "intrinsic id is null";
+            CallSite cs = (site != null) ? site : scopes.peek();
+            assert cs != null : "no CallSite?";
+            cs.setIntrinsicName(id);
         } else if (qname.equals("regalloc")) {
             compile.setAttempts(Integer.parseInt(search(atts, "attempts")));
         } else if (qname.equals("inline_fail")) {
