@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -23,7 +23,7 @@
 
 /*
  * @test
- * @bug 4981566 5028634 5094412 6304984 7025786 7025789 8001112 8028545 8000961 8030610 8028546 8188870 8173382 8173382
+ * @bug 4981566 5028634 5094412 6304984 7025786 7025789 8001112 8028545 8000961 8030610 8028546 8188870 8173382 8173382 8245147
  * @summary Check interpretation of -target and -source options
  * @modules java.compiler
  *          jdk.compiler
@@ -41,8 +41,15 @@ import javax.tools.StandardJavaFileManager;
 import java.util.List;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Set;
+import java.util.function.BiConsumer;
+import java.util.function.Consumer;
 
-
+/*
+ * If not explicitly specified the latest source and latest target
+ * values are the defaults. If explicitly specified, the target value
+ * has to be greater than or equal to the source value.
+ */
 public class Versions {
 
     protected JavaCompiler javacompiler;
@@ -59,80 +66,115 @@ public class Versions {
         versions.run();
     }
 
-    void run() {
+    public static final Set<String> RETIRED_SOURCES =
+        Set.of("1.2", "1.3", "1.4", "1.5");
 
+    public static final Set<String> VALID_SOURCES =
+        Set.of("1.6", "1.7", "1.8", "1.9", "1.10", "11");
+
+    public static final String LATEST_MAJOR_VERSION = "55.0";
+
+    static enum SourceTarget {
+        SIX(true,     "50.0",  "6", Versions::checksrc6),
+        SEVEN(true,   "51.0",  "7", Versions::checksrc7),
+        EIGHT(true,   "52.0",  "8", Versions::checksrc8),
+        NINE(true,    "53.0",  "9", Versions::checksrc9),
+        TEN(true,     "54.0", "10", Versions::checksrc10),
+        ELEVEN(false, "55.0", "11", Versions::checksrc11);
+
+        private final boolean dotOne;
+        private final String classFileVer;
+        private final String target;
+        private final BiConsumer<Versions, List<String>> checker;
+
+        private SourceTarget(boolean dotOne, String classFileVer, String target,
+                             BiConsumer<Versions, List<String>> checker) {
+            this.dotOne = dotOne;
+            this.classFileVer = classFileVer;
+            this.target = target;
+            this.checker = checker;
+        }
+
+        public void checksrc(Versions version, List<String> args) {
+            checker.accept(version, args);
+        }
+
+        public boolean dotOne() {
+            return dotOne;
+        }
+
+        public String classFileVer() {
+            return classFileVer;
+        }
+
+        public String target() {
+            return target;
+        }
+    }
+
+    void run() {
         String TC = "";
         System.out.println("Version.java: Starting");
 
-        check("55.0");
-        check("55.0", "-source 1.6");
-        check("55.0", "-source 1.7");
-        check("55.0", "-source 1.8");
-        check("55.0", "-source 1.9");
-        check("55.0", "-source 1.10");
-        check("55.0", "-source 11");
+        check(LATEST_MAJOR_VERSION);
+        for (String source : VALID_SOURCES) {
+            check(LATEST_MAJOR_VERSION, List.of("-source " + source));
+        }
 
-        check_source_target(true, "50.0", "6", "6");
-        check_source_target(true, "51.0", "6", "7");
-        check_source_target(true, "51.0", "7", "7");
-        check_source_target(true, "52.0", "6", "8");
-        check_source_target(true, "52.0", "7", "8");
-        check_source_target(true, "52.0", "8", "8");
-        check_source_target(true, "53.0", "6", "9");
-        check_source_target(true, "53.0", "7", "9");
-        check_source_target(true, "53.0", "8", "9");
-        check_source_target(true, "53.0", "9", "9");
-        check_source_target(true, "54.0", "6", "10");
-        check_source_target(true, "54.0", "7", "10");
-        check_source_target(true, "54.0", "8", "10");
-        check_source_target(true, "54.0", "9", "10");
-        check_source_target(true, "54.0", "10", "10");
-        check_source_target(false, "55.0", "6", "11");
-        check_source_target(false, "55.0", "7", "11");
-        check_source_target(false, "55.0", "8", "11");
-        check_source_target(false, "55.0", "9", "11");
-        check_source_target(false, "55.0", "10", "11");
-        check_source_target(false, "55.0", "11", "11");
+        // Verify that a -source value less than a -target value is
+        // accepted and that the resulting class files are dependent
+        // on the target setting alone.
+        SourceTarget[] sourceTargets = SourceTarget.values();
+        for (int i = 0; i < sourceTargets.length; i++) {
+            SourceTarget st = sourceTargets[i];
+            String classFileVer = st.classFileVer();
+            String target = st.target();
+            boolean dotOne = st.dotOne();
+            check_source_target(dotOne, List.of(classFileVer, target, target));
+            for (int j = i - 1; j >= 0; j--) {
+                String source = sourceTargets[j].target();
+                check_source_target(dotOne, List.of(classFileVer, source, target));
+            }
+        }
 
-        checksrc16("-source 1.6");
-        checksrc16("-source 6");
-        checksrc16("-source 1.6", "-target 1.6");
-        checksrc16("-source 6", "-target 6");
-        checksrc17("-source 1.7");
-        checksrc17("-source 7");
-        checksrc17("-source 1.7", "-target 1.7");
-        checksrc17("-source 7", "-target 7");
-        checksrc18("-source 1.8");
-        checksrc18("-source 8");
-        checksrc18("-source 1.8", "-target 1.8");
-        checksrc18("-source 8", "-target 8");
-        checksrc19("-source 1.9");
-        checksrc19("-source 9");
-        checksrc19("-source 1.9", "-target 1.9");
-        checksrc19("-source 9", "-target 9");
-        checksrc110();
-        checksrc110("-source 1.10");
-        checksrc110("-source 10");
-        checksrc110("-source 1.10", "-target 1.10");
-        checksrc110("-source 10", "-target 10");
-        checksrc111("-source 11");
-        checksrc111("-source 11", "-target 11");
-        checksrc111("-target 11");
+        // Verify acceptance of different combinations of -source N,
+        // -target M; N <= M
+        for (int i = 0; i < sourceTargets.length; i++) {
+            SourceTarget st = sourceTargets[i];
 
-        fail("-source 7", "-target 1.6", "Base.java");
-        fail("-source 8", "-target 1.6", "Base.java");
-        fail("-source 8", "-target 1.7", "Base.java");
-        fail("-source 9", "-target 1.7", "Base.java");
-        fail("-source 9", "-target 1.8", "Base.java");
-        fail("-source 10", "-target 1.7", "Base.java");
-        fail("-source 10", "-target 1.8", "Base.java");
-        fail("-source 11", "-target 1.9", "Base.java");
-        fail("-source 11", "-target 1.10", "Base.java");
+            st.checksrc(this, List.of("-source " + st.target()));
+            st.checksrc(this, List.of("-source " + st.target(), "-target " + st.target()));
 
-        fail("-source 1.5", "-target 1.5", "Base.java");
-        fail("-source 1.4", "-target 1.4", "Base.java");
-        fail("-source 1.3", "-target 1.3", "Base.java");
-        fail("-source 1.2", "-target 1.2", "Base.java");
+            if (st.dotOne()) {
+                st.checksrc(this, List.of("-source 1." + st.target()));
+                st.checksrc(this, List.of("-source 1." + st.target(), "-target 1." + st.target()));
+            }
+
+            if (i == sourceTargets.length - 1) {
+                // Can use -target without -source setting only for
+                // most recent target since the most recent source is
+                // the default.
+                st.checksrc(this, List.of("-target " + st.target()));
+
+                if (!st.classFileVer().equals(LATEST_MAJOR_VERSION)) {
+                    throw new RuntimeException(st +
+                                               "does not have class file version" +
+                                               LATEST_MAJOR_VERSION);
+                }
+            }
+        }
+
+        // Verify that -source N -target (N-1) is rejected
+        for (int i = 1 /* Skip zeroth value */; i < sourceTargets.length; i++) {
+            fail(List.of("-source " + sourceTargets[i].target(),
+                 "-target " + sourceTargets[i-1].target(),
+                         "Base.java"));
+        }
+
+        // Previously supported source/target values
+        for (String source  : RETIRED_SOURCES) {
+            fail(List.of("-source " + source, "-target " + source, "Base.java"));
+        }
 
         if (failedCases > 0) {
             System.err.println("failedCases = " + String.valueOf(failedCases));
@@ -141,7 +183,7 @@ public class Versions {
 
     }
 
-    protected void printargs(String fname,String... args) {
+    protected void printargs(String fname, List<String> args) {
         System.out.printf("test: %s", fname);
         for (String onearg : args) {
             System.out.printf(" %s", onearg);
@@ -149,22 +191,26 @@ public class Versions {
         System.out.printf("\n", fname);
     }
 
-    protected void check_source_target(boolean dotOne, String... args) {
+    protected void check_source_target(boolean dotOne, List<String> args) {
         printargs("check_source_target", args);
-        check_target(dotOne, args[0], args[1], args[2]);
+        check_target(dotOne, List.of(args.get(0), args.get(1), args.get(2)));
         if (dotOne) {
-            check_target(dotOne, args[0], "1." + args[1], args[2]);
+            check_target(dotOne, List.of(args.get(0), "1." + args.get(1), args.get(2)));
         }
     }
 
-    protected void check_target(boolean dotOne, String... args) {
-        check(args[0], "-source " + args[1], "-target " + args[2]);
+    protected void check_target(boolean dotOne, List<String> args) {
+        check(args.get(0), List.of("-source " + args.get(1), "-target " + args.get(2)));
         if (dotOne) {
-            check(args[0], "-source " + args[1], "-target 1." + args[2]);
+            check(args.get(0), List.of("-source " + args.get(1), "-target 1." + args.get(2)));
         }
     }
 
-    protected void check(String major, String... args) {
+    protected void check(String major) {
+        check(major, List.of());
+    }
+
+    protected void check(String major, List<String> args) {
         printargs("check", args);
         List<String> jcargs = new ArrayList<>();
         jcargs.add("-Xlint:-options");
@@ -190,68 +236,65 @@ public class Versions {
         }
     }
 
-    protected void checksrc16(String... args) {
-        printargs("checksrc16", args);
-        int asize = args.length;
-        String[] newargs = new String[asize + 1];
-        System.arraycopy(args, 0, newargs, 0, asize);
-        newargs[asize] = "Base.java";
-        pass(newargs);
-        newargs[asize] = "New17.java";
-        fail(newargs);
+    protected void checksrc6(List<String> args) {
+        printargs("checksrc6", args);
+        expectedPass(args, List.of("Base.java"));
+        expectedFail(args, List.of("New7.java"));
     }
 
-    protected void checksrc17(String... args) {
-        printargs("checksrc17", args);
-        int asize = args.length;
-        String[] newargs = new String[asize+1];
-        System.arraycopy(args, 0, newargs,0 , asize);
-        newargs[asize] = "New17.java";
-        pass(newargs);
-        newargs[asize] = "New18.java";
-        fail(newargs);
+    protected void checksrc7(List<String> args) {
+        printargs("checksrc7", args);
+        expectedPass(args, List.of("New7.java"));
+        expectedFail(args, List.of("New8.java"));
     }
 
-    protected void checksrc18(String... args) {
-        printargs("checksrc18", args);
-        int asize = args.length;
-        String[] newargs = new String[asize+1];
-        System.arraycopy(args, 0, newargs,0 , asize);
-        newargs[asize] = "New17.java";
-        pass(newargs);
-        newargs[asize] = "New18.java";
-        pass(newargs);
-        newargs[asize] = "New110.java";
-        fail(newargs);
+    protected void checksrc8(List<String> args) {
+        printargs("checksrc8", args);
+        expectedPass(args, List.of("New7.java", "New8.java"));
+        expectedFail(args, List.of("New10.java"));
     }
 
-    protected void checksrc19(String... args) {
-        printargs("checksrc19", args);
-        checksrc18(args);
+    protected void checksrc9(List<String> args) {
+        printargs("checksrc9", args);
+        expectedPass(args, List.of("New7.java", "New8.java"));
+        expectedFail(args, List.of("New10.java"));
     }
 
-    protected void checksrc110(String... args) {
-        printargs("checksrc110", args);
-        int asize = args.length;
-        String[] newargs = new String[asize+1];
-        System.arraycopy(args, 0, newargs,0 , asize);
-        newargs[asize] = "New17.java";
-        pass(newargs);
-        newargs[asize] = "New18.java";
-        pass(newargs);
-        newargs[asize] = "New110.java";
-        pass(newargs);
+    protected void checksrc10(List<String> args) {
+        printargs("checksrc10", args);
+        expectedPass(args, List.of("New7.java", "New8.java", "New10.java"));
+        expectedFail(args, List.of("New11.java"));
     }
 
-    protected void checksrc111(String... args) {
-        printargs("checksrc111", args);
-        checksrc110(args);
+    protected void checksrc11(List<String> args) {
+        printargs("checksrc11", args);
+        expectedPass(args, List.of("New7.java", "New8.java", "New10.java", "New11.java"));
+        expectedFail(args, List.of("New14.java"));
     }
 
-    protected void pass(String... args) {
+    protected void expected(List<String> args, List<String> fileNames,
+                            Consumer<List<String>> passOrFail) {
+        ArrayList<String> fullArguments = new ArrayList<>(args);
+        // Issue compile with each filename in turn.
+        for(String fileName : fileNames) {
+            fullArguments.add(fileName);
+            passOrFail.accept(fullArguments);
+            fullArguments.remove(fullArguments.size() - 1);
+        }
+    }
+
+    protected void expectedPass(List<String> args, List<String> fileNames) {
+        expected(args, fileNames, this::pass);
+    }
+
+    protected void expectedFail(List<String> args, List<String> fileNames) {
+        expected(args, fileNames, this::fail);
+    }
+
+    protected void pass(List<String> args) {
         printargs("pass", args);
 
-        List<String> jcargs = new ArrayList<String>();
+        List<String> jcargs = new ArrayList<>();
         jcargs.add("-Xlint:-options");
 
         // add in args conforming to List requrements of JavaCompiler
@@ -286,10 +329,10 @@ public class Versions {
 
     }
 
-    protected void fail(String... args) {
+    protected void fail(List<String> args) {
         printargs("fail", args);
 
-        List<String> jcargs = new ArrayList<String>();
+        List<String> jcargs = new ArrayList<>();
         jcargs.add("-Xlint:-options");
 
         // add in args conforming to List requrements of JavaCompiler
@@ -318,11 +361,13 @@ public class Versions {
             System.err.println("fail: Compilation erroneously succeeded");
             System.err.println("\t arguments:\t" + jcargs);
             System.err.println("\t file     :\t" + filename);
+            System.out.println("GLGLGL!!! 6");
+            System.err.println("GLGLGL!!! 6");
             failedCases++;
         }
     }
 
-    protected boolean compile(String sourceFile, List<String>options) {
+    protected boolean compile(String sourceFile, List<String> options) {
         JavaCompiler.CompilationTask jctask;
         try (StandardJavaFileManager fm = javacompiler.getStandardFileManager(null, null, null)) {
             Iterable<? extends JavaFileObject> files = fm.getJavaFileObjects(sourceFile);
@@ -357,19 +402,19 @@ public class Versions {
         writeSourceFile("Base.java","public class Base { }\n");
 
         /*
-         * Create a file with a new feature in 1.7, not in 1.6 : "<>"
+         * Create a file with a new feature in 7, not in 6 : "<>"
          */
-        writeSourceFile("New17.java",
+        writeSourceFile("New7.java",
             "import java.util.List;\n" +
             "import java.util.ArrayList;\n" +
-            "class New17 { List<String> s = new ArrayList<>(); }\n"
+            "class New7 { List<String> s = new ArrayList<>(); }\n"
         );
 
         /*
-         * Create a file with a new feature in 1.8, not in 1.7 : lambda
+         * Create a file with a new feature in 8, not in 7 : lambda
          */
-        writeSourceFile("New18.java",
-            "public class New18 { \n" +
+        writeSourceFile("New8.java",
+            "public class New8 { \n" +
             "    void m() { \n" +
             "    new Thread(() -> { }); \n" +
             "    } \n" +
@@ -377,16 +422,44 @@ public class Versions {
         );
 
         /*
-         * Create a file with a new feature in 1.10, not in 1.9 : var
+         * Create a file with a new feature in 10, not in 9 : var
          */
-        writeSourceFile("New110.java",
-            "public class New110 { \n" +
+        writeSourceFile("New10.java",
+            "public class New10 { \n" +
             "    void m() { \n" +
             "    var tmp = new Thread(() -> { }); \n" +
             "    } \n" +
             "} \n"
         );
 
+        /*
+         * Create a file with a new feature in 11, not in 10 : var for lambda parameters
+         */
+        writeSourceFile("New11.java",
+            "public class New11 { \n" +
+            "    static java.util.function.Function<String,String> f = (var x) -> x.substring(0); \n" +
+            "    void m(String name) { \n" +
+            "    var tmp = new Thread(() -> { }, f.apply(name)); \n" +
+            "    } \n" +
+            "} \n"
+        );
+
+        /*
+         * Create a file with a new feature in 14, not in 13 : switch expressions
+         */
+        writeSourceFile("New14.java",
+            "public class New14 { \n" +
+            "    static { \n" +
+            "        int i = 5; \n" +
+            "        System.out.println( \n" +
+            "            switch(i) { \n" +
+            "                case 0 -> false; \n" +
+            "                default -> true; \n" +
+            "            } \n" +
+            "        ); \n" +
+            "    } \n" +
+            "} \n"
+        );
     }
 
     protected boolean checkClassFileVersion
