@@ -22,13 +22,31 @@
  */
 
 /**
- * JDK-8048079: Persistent code store is broken after optimistic types merge
- * Same script as JDK-8048079_1b.js to exercise code cache again.
+ * Array extension check
+ *
  * @test
- * @runif external.yui
- * @option -pcc
- * @option -Dnashorn.persistent.code.cache=build/nashorn_code_cache
- * @fork
+ * @run
  */
 
-load(__DIR__ + 'yui.js');
+"use strict";
+var a = [1,2,3];
+Object.preventExtensions(a);
+try {
+    a[4] = 4;
+    print(a);
+} catch (e) {
+    if (!(e instanceof TypeError)) {
+	print("TypeError expected but got e");
+    }
+}
+
+if (a[0] != 1) {
+    throw "element 0 is wrong";
+}
+if (a[1] != 2) {
+    throw "element 1 is wrong";
+}
+if (a[2] != 3) {
+    throw "element 2 is wrong";
+}
+
